using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace EggTimer
{
    public partial class EggTimer : Form
    {
        private ArrayList watches;

        private delegate void NotifyTimerEndDelagate(string name);

        public EggTimer()
        {
            InitializeComponent();
            watches = new ArrayList();
        }

        private void UpdateTime(object sender, EventArgs e)
        {
            string days = string.Format("{0:0}", daysUpDown.Value);
            string hours = string.Format("{0:00}", hoursUpDown.Value);
            string mins = string.Format("{0:00}", minsUpDown.Value);
            string secs = string.Format("{0:00}", secsUpDown.Value);

            timerLabel.Text = days + ":" + hours + ":" + mins + ":" + secs;
        }

        private void Up1Hour(object sender, EventArgs e)
        {
            hoursUpDown.Value += 1;
        }

        private void Up4Hours(object sender, EventArgs e)
        {
            hoursUpDown.Value += 4;
        }

        private void Up1Min(object sender, EventArgs e)
        {
            minsUpDown.Value += 1;
        }

        private void Up5Mins(object sender, EventArgs e)
        {
            minsUpDown.Value += 5;
        }

        private void Up10Mins(object sender, EventArgs e)
        {
            minsUpDown.Value += 10;
        }

        private void Up1Sec(object sender, EventArgs e)
        {
            secsUpDown.Value += 1;
        }

        private void Up5Secs(object sender, EventArgs e)
        {
            secsUpDown.Value += 5;
        }

        private void Up10Secs(object sender, EventArgs e)
        {
            secsUpDown.Value += 10;
        }

        private void UpdateWatches(object sender, EventArgs e)
        {
            for (int i = watches.Count - 1; i >= 0; i--)
            {
                Watch watch = (Watch)watches[i];
                watch.Tick();

                if (watch.Ended && !watch.Visible)
                {
                    notifyIcon.Visible = true;
                    Invoke(new NotifyTimerEndDelagate(NotifyTimerEnd), watch.Text);
                    watches.RemoveAt(i);
                }
            }
        }

        private void NotifyTimerEnd(string name)
        {
            notifyIcon.ShowBalloonTip(3000, "Egg Timer", string.Format("Timer {0} was elapsed!", name), ToolTipIcon.Info);
        }

        private void CreateClick(object sender, EventArgs e)
        {
            string name = nameEdit.Text.Trim();

            if (name.Equals(""))
            {
                MessageBox.Show("Name is required", "Required", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            int estimated = ((int)daysUpDown.Value) * 86400 + ((int)hoursUpDown.Value) * 3600 + ((int)minsUpDown.Value) * 60 + ((int)secsUpDown.Value);

            if (estimated <= 0)
            {
                MessageBox.Show("Estimated time is invalid", "Invalid", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            foreach (Watch w in watches)
            {
                if (name.Equals(w.Text))
                {
                    MessageBox.Show(string.Format("The timer '{0}' already exists", name), "Duplicated", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }

            Watch watch = new Watch(name, estimated);
            watch.FormClosed += new FormClosedEventHandler(TimerClosedEvent);
            watches.Add(watch);
            watch.Show();

            timer.Enabled = true;
        }

        void TimerClosedEvent(object sender, FormClosedEventArgs e)
        {
            watches.Remove(sender);
        }

        private void FormClosingEvent(object sender, FormClosingEventArgs e)
        {
            // this.Hide();
            // e.Cancel = true;
        }

        private void ResetClick(object sender, EventArgs e)
        {
            daysUpDown.Value = 0;
            hoursUpDown.Value = 0;
            minsUpDown.Value = 0;
            secsUpDown.Value = 0;
            nameEdit.Text = "";
        }

        private void HomePageClickEvent(object sender, EventArgs e)
        {
            System.Diagnostics.Process process = new System.Diagnostics.Process();
            process.StartInfo.FileName = "http://www.ravewiki.com/wiki/Egg Timer";
            process.Start();
        }

        private void QuitClickEvent(object sender, EventArgs e)
        {
            FormClosing -= FormClosingEvent;
            Application.Exit();
        }

        private void TrayDoubleClickEvent(object sender, MouseEventArgs e)
        {
            this.Show();
            this.Activate();
        }

        private void NewTimerMenuClick(object sender, EventArgs e)
        {
            this.Show();
            this.Activate();
        }

        private void MenuOpeningEvent(object sender, CancelEventArgs e)
        {
            ToolStripItemCollection items = notifyMenu.Items;

            for (int i = items.Count - 1; i >= 0; i--)
            {
                if (items[i].Tag is Watch)
                    items.RemoveAt(i);
            }

            foreach (Watch watch in watches)
            {
                ToolStripMenuItem item = new ToolStripMenuItem(watch.Text);
                item.Tag = watch;
                items.Insert(2, item);

                if (!watch.Visible)
                {
                    ToolStripMenuItem showMenu = new ToolStripMenuItem("Show");
                    showMenu.Click += ShowMenuClick;
                    showMenu.Tag = watch;
                    item.DropDownItems.Add(showMenu);
                }

                ToolStripMenuItem playMenu = new ToolStripMenuItem("Play");
                if (watch.Status != Status.Playing)
                {
                    playMenu.Click += PlayMenuClick;
                    playMenu.Tag = watch;
                }
                else
                    playMenu.Enabled = false;

                ToolStripMenuItem pauseMenu = new ToolStripMenuItem("Pause");
                if (watch.Status == Status.Playing)
                {
                    pauseMenu.Click += PauseMenuClick;
                    pauseMenu.Tag = watch;
                }
                else
                    pauseMenu.Enabled = false;

                ToolStripMenuItem stopMenu = new ToolStripMenuItem("Stop");
                if (watch.Status != Status.Stopped)
                {
                    stopMenu.Click += StopMenuClick;
                    stopMenu.Tag = watch;
                }
                else
                    stopMenu.Enabled = false;

                item.DropDownItems.Add(playMenu);
                item.DropDownItems.Add(pauseMenu);
                item.DropDownItems.Add(stopMenu);
            }
        }

        private void ShowMenuClick(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            Watch watch = (Watch)item.Tag;
            watch.Show();
        }

        private void PauseMenuClick(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            Watch watch = (Watch)item.Tag;
            watch.PauseWatch();
        }

        private void StopMenuClick(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            Watch watch = (Watch)item.Tag;
            watch.StopWatch();
        }

        private void PlayMenuClick(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            Watch watch = (Watch)item.Tag;
            watch.PlayWatch();
        }

        private void btnQuail_Click(object sender, EventArgs e)
        {
            ADLab.Quail.Engine.RunDesigner(Path.Combine(Application.StartupPath, "workflow2.xoml"), false);
        }
    }
}